from splverifier.api.base import console
from splverifier.api.config import PATHS
from splverifier.api.statistics import Writer
from splverifier.api.fs import *
from os.path import dirname, basename
import os

class Experiment(object):

    def __init__(self, outdir, spl):
        self.spl = spl
        self.fs = FileStore(outdir)
        if self.fs.exists():
            self.fs.clear().create()
    
    def check_against_all_specifications(self, variants=True, ve=True):
        console.header('Checking all interactions')
        for interaction in self.spl.interactions:
            console.header('Checking interaction %d' % interaction[0])
            self.spl.fs = self.fs.get_fs(self.get_interaction_dir(interaction)).create()
            
            self.writer = Writer(self.spl.fs.get('%d.log' % interaction[0]), mode='a')
            specification = self.spl.containment_hierarchy + interaction[1]
            scenario = self.spl.scenario_config.scenarios[specification]
            if ve:
                self.check_variability_encoded_spl(specification, scenario)
            if variants:
                self.check_variants(specification, scenario)
            
    
    def check_variants(self, specification, scenario):
        applicable = self.spl.get_applicable_variants(specification)
        console.info('Applicable variants :\n\t' + '\n\t'.join(applicable))
        for feature_selection in applicable:
            self.check_variant(feature_selection, specification, scenario)

    def _prepare_variant(self, feature_selection, specification, scenario):
        console.header('+Preparing %s' % feature_selection)
        console.info('-Composing')
        variant = self.spl.compose(feature_selection)
        console.info('-Adding scenario')
        variant.add_scenario(scenario, action_dict=self.spl.scenario_config.action_map)
        console.info('-Weaving specification')
        variant.weave_specification(specification)
        return variant

    def check_variant(self, feature_selection, specification, scenario):
        variant = self._prepare_variant(feature_selection, specification, scenario)
        console.header('+Checking %s' % feature_selection)        
        self.checker.check(variant)
        self.writer.write(
            variant.num,
            variant.composition_time,
            variant.weaving_time,
            variant.preparation_time,
            variant.check_time,
            (
                variant.composition_time + 
                variant.weaving_time + 
                variant.preparation_time +
                variant.check_time
            ), #total
            0, #lifted
            variant.check_result,
        )


    def run_variant(self, feature_selection, specification, scenario):
        variant = self._prepare_variant(feature_selection, specification, scenario)
        console.header('+Compiling %s' % feature_selection)
        excludes = ['wsllib_check.c']
        files = [x for x in os.listdir(variant.checkable) if x.endswith('.c') and x not in excludes]
        system('cd %s;gcc %s' %(variant.checkable, ' '.join(files)))
        system('cd %s;./a.out' %(variant.checkable, ))
        
    def check_variability_encoded_spl(self, specification, scenario):
        console.header('+Preparing variablity encoded SPL')
        console.info('-Composing')
        specification_feature = os.path.basename(os.path.dirname(specification))
        variant = self.spl.encode_variability(assume_features=[specification_feature])
        console.info('-Adding scenario')
        variant.add_scenario(scenario, action_dict=self.spl.scenario_config.action_map)
        
        console.info('-Weaving specification')
        variant.weave_specification(specification)
        console.info('-Checking')
        self.checker.check(variant)
        console.info('Checking done: ' + variant.check_result)
        self.writer.write(
            variant.num,
            variant.composition_time,
            variant.weaving_time,
            variant.preparation_time,
            variant.check_time,
            (
                variant.composition_time + 
                variant.weaving_time + 
                variant.preparation_time +
                variant.check_time
            ), #total
            1, #lifted
            variant.check_result,
        )

        
    def get_interaction_dir(self, interaction):
        return "%d_%s" % (interaction[0], nameonly(interaction[1]))


