from splverifier.api.base import *
from splverifier.api.fs import *
from splverifier.api.config import *
from splverifier.integration import featurehouse
from splverifier.integration import featurescene
from splverifier.integration import autofeature
from os.path import dirname, basename

class Variant(Expando):
    checkable = None
    variability_encoded = False
    num = None
    location = None
    feature_selection = None
    composition_time = -1
    weaving_time = -1
    

    def add_scenario(self, scenario, action_dict):
        featurescene.generate_scenario(
            scenario,
            outfile = Path(self.location, 'scenario.c'),
            expression = self.feature_selection,
            variability_encode = self.variability_encoded,
            action_dict = action_dict
        )
        self.scenario = scenario
        
    def weave_specification(self, specification):
        checkable = Path(self.location, 'woven')
        self.weaving_time = autofeature.weave_specifications(
            self.location,
            [specification],
            checkable
        )
        self.checkable = checkable

    
class ProductLine(Expando):
    """
    Abstract base for specific SPLs e.g. EmailSystem
    
    each SPL should define a model file in guidsl format
    """
    
    specifications=[]
    
    features=[]
    
    containment_hierarchy=''
        
    def compose(self, feature_selection, target_dir=None):
        
        if not target_dir:
            self.fs.create('variants')
            target_dir = self.fs.get('variants', nameonly(feature_selection))
        
        composition_time = featurehouse.compose(
            feature_selection,
            self.containment_hierarchy,
            target_dir = target_dir,
        )
        
        return Variant(
            num = nameonly(feature_selection),
            location = target_dir,
            feature_selection = feature_selection,
            composition_time = composition_time,
            variability_encoded = False,
        )
    
    def encode_variability(self, assume_features=None, target=None):
        
        if assume_features is None:
            assume_features = []
        
        if not target:
            target = self.fs.get('variability_encoded')
        
        composition_time = featurehouse.encode_variability(
            self.feature_model.all_expression, 
            self.containment_hierarchy,
            target,
            self.feature_model.cnf_file,
            assume_features=assume_features
        )
        
        return Variant(
            num = -1,
            location = target,
            feature_selection = self.feature_model.all_expression,
            composition_time = composition_time,
            variability_encoded = True,
        )
    
    def generate_scenario(self, scenario, outfile=None, selected_features=None, variability_encode=False, action_dict=None):
        
        featurescene.generate_scenario(scenario, outfile, expression=selected_features, variability_encode=variability_encode, action_dict=action_dict)
        
        

    def get_applicable_variants(self, specification):
        associated_feature = basename(dirname(specification))
        return [variant for variant in self.variants if associated_feature in featurehouse.get_features_from_expression(variant)]

    def get_scenario_for_specification(self, specification):
        raise

