from splverifier.api import modelchecker
from splverifier.api.fs import *
from splverifier.api.config import PATHS
from splverifier.integration.cilly import cilly
import time
from datetime import datetime, timedelta
import subprocess
import glob
import os


def formattime(delta):
    return str(delta.seconds) + '.' + str(delta.microseconds).rjust(6, '0')

def wait_timeout(cmd, seconds):
    print cmd
    
    proc = subprocess.Popen(cmd, shell=True)
    #http://stackoverflow.com/questions/1359383/python-run-a-process-and-kill-it-if-it-doesnt-end-within-one-hour
    """Wait for a process to finish, or raise exception after timeout"""
    start = time.time()
    end = start + seconds
    interval = min(seconds / 1000.0, .25)

    while True:
        result = proc.poll()
        if result is not None:
            return result
        if time.time() >= end:
            proc.kill()
            raise RuntimeError("Process timed out")
        time.sleep(interval)

class CPACheckerResult(object):
    cil_time = -1
    cpa_time = -1
    
    @property
    def check_time(self):
        return self.cil_time + self.cpa_time



class CPAchecker(modelchecker.ModelChecker):

    statsfile = '/home/henne/splvs.log'
    logfile = PATHS.cpachecker.cpachecker_dir + '/test/output/full_log.txt'
    configfile = 'test/config/symbpredabsCPA-bmc.properties'
    #configfile = 'test/config/explicitAnalysisInf.properties'
    
    def __init__(self):
        modelchecker.ModelChecker.__init__(self)
    
    def do_check(self, variant):
        resultfs = FileStore(variant.location, 'result_CPAchecker').create()
        if not hasattr(variant, 'cilfile'):
            self.transform_to_cil(variant)
        variant.check_result, variant.check_time = self.check_cilfile(
            variant.cilfile,
            resultfs.get(),
        )
        return variant

    def transform_to_cil(self, variant):
        cilfs = FileStore(variant.location, 'cil').create()
        cilfile = cilfs.get('a.cil.c')
        variant.preparation_time = cilly(
            variant.checkable,
            cilfile,
            excludes = ['wsllib.c'],
        )
        variant.cilfile = cilfile


    def check_cilfile(self, cilfile, outdir):
        timer = Timer().start()
        try:
            wait_timeout(
                'cd %s;ulimit -v %s;scripts/cpa.sh \
                    -config %s \
                     %s 2>&1 | tee %s' % (
                        PATHS.cpachecker.cpachecker_dir,
                        PATHS.cpachecker.ulimit_v,
                        self.configfile,
                        cilfile,
                        self.logfile
                     ),
                PATHS.cpachecker.timeout
            )
        except RuntimeError:
            system('pkill -f "java -Djava.library.path=scripts*"')
        
        timer.stop()
        
        time.sleep(5)
        result = self.extract_result(self.logfile)
        
        system('cd %s;mv test/output %s' % (PATHS.cpachecker.cpachecker_dir, outdir))
        system('cd %s;mkdir -p test/output' % (PATHS.cpachecker.cpachecker_dir,))
        return (result, timer.elapsed)
        
    def extract_result(self, logfile):
        contents = open(logfile).read()
        
        if 'YES, there is a BUG!' in contents:
            return 'BUG'
            
        elif 'NO, the system is considered safe by the chosen CPAs' in contents:
            return 'SAFE'
        
        else:
            return 'OOM'
            

class DeferredCPAchecker(CPAchecker):

    def check_cilfile(self, cilfile, outdir):
        
        return ('NA', timedelta(-1))

    def do_check(self, variant):
        variant = CPAchecker.do_check(self, variant)
        import json
        data = json.dumps(dict(
            num = variant.num,
            composition_time = float(formattime(variant.composition_time)),
            weaving_time = float(formattime(variant.weaving_time)),
            preparation_time = float(formattime(variant.preparation_time)),
            variability_encoded = variant.variability_encoded, #lifted
        ), indent=4)
        variant.serialized = data
        return variant
